<?php

namespace Drupal\tmgmt_contentapi\Services;

use Drupal\tmgmt_contentapi\Swagger\Client\Api\StatusUpdateApi;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\TokenApi;
use Drupal\tmgmt_contentapi\Swagger\Client\Model\CreateToken;
use Drupal\tmgmt_contentapi\Util\ConentApiHelper;
use Drupal\tmgmt\Entity\Job;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\RequestApi;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\FileApi;
use Drupal\tmgmt_contentapi\Swagger\Client\Model\ArrayOfRequestIds;
use Drupal\tmgmt_contentapi\Swagger\Client\Configuration;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\File\FileRepositoryInterface;
use Symfony\Component\Mime\MimeTypeGuesserInterface;
use Drupal\file\Entity\File;
use Drupal\tmgmt_contentapi\Plugin\tmgmt_contentapi\Format\Xliff;
use Drupal\tmgmt_contentapi\Util\GeneralHelper;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\tmgmt\TranslatorInterface;
use Drupal\tmgmt_contentapi\Swagger\Client\Api\JobApi;

/**
 * Service to process job.
 *
 * Acknowledge, update, delete jobs.
 *
 * @category Class
 * @package Drupal\tmgmt_contentapi\Services
 * @author Lionbridge team
 */
class ProcessJob {
  /**
   * The job id which nees to processed.
   *
   * @var string
   */
  protected $filterJobId;

  /**
   * The capi jobs array.
   *
   * @var array
   */
  protected $capiJobs;

  /**
   * The ctt jobs array.
   *
   * @var array
   */
  protected $jobsCtt;

  /**
   * Status api object.
   *
   * @var Drupal\tmgmt_contentapi\Swagger\Client\Api\StatusUpdateApi
   */
  protected $statusApi;

  /**
   * The capi v1 jobs array.
   *
   * @var array
   */
  protected $capiV1Jobs;

  /**
   * The capi v2 jobs array.
   *
   * @var array
   */
  protected $capiV2Jobs;

  /**
   * The capi token.
   *
   * @var string
   */
  protected $capiToken;

  /**
   * The ctt token.
   *
   * @var string
   */
  protected $cttToken;

  /**
   * The translator object.
   *
   * @var \Drupal\tmgmt\TranslatorInterface
   */
  protected $translator;

  /**
   * Token API object.
   *
   * @var \Drupal\tmgmt_contentapi\Swagger\Client\Api\TokenApi
   */
  protected $capi;

  /**
   * The capi job which are ready for deliver.
   *
   * @var array
   */
  protected $capiToDeliver;

  /**
   * The capi job with status cancelled.
   *
   * @var array
   */
  protected $capiCancelled;

  /**
   * The capi job which are not yet ready for deliver.
   *
   * @var array
   */
  protected $capiToIgnore;

  /**
   * The request api.
   *
   * @var \Drupal\tmgmt_contentapi\Swagger\Client\Api\RequestApi
   */
  protected $requestApi;

  /**
   * The request api.
   *
   * @var \Drupal\tmgmt_contentapi\Swagger\Client\Api\FileApi
   */
  protected $fileApi;

  /**
   * The request api.
   *
   * @var \Drupal\tmgmt_contentapi\Swagger\Client\Model\ArrayOfRequestIds
   */
  protected $arrayOfRequestId;

  /**
   * Public file path for received files.
   */
  const PUBLIC_RECEIVED_FILE_PATH = '://tmgmt_contentapi/LioxReceivedFiles/';

  /**
   * The request api.
   *
   * @var \Drupal\tmgmt_contentapi\Swagger\Client\Configuration
   */
  protected $config;

  /**
   * The filesystem service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The file repository service.
   *
   * @var \Drupal\File\FileRepositoryInterface
   */
  protected $fileRepository;

  /**
   * The stream wrapper service.
   *
   * @var use\Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected $streamWrapperManager;

  /**
   * If provider allows to auto accept jobs.
   *
   * @var bool
   */
  protected $isAutoAccept;

  /**
   * Check if auto import is on.
   *
   * @var bool
   */
  protected $isAutoImport;

  /**
   * The stream wrapper service.
   *
   * @var \Symfony\Component\Mime\MimeTypeGuesserInterface
   */
  protected $guesser;

  /**
   * The request api.
   *
   * @var \Drupal\tmgmt_contentapi\Swagger\Client\Api\JobApi
   */
  protected $jobApi;

  /**
   * The filesystem service.
   *
   * @var \Drupal\Core\Datetime\DrupalDateTime
   */
  protected $drupalDateTime;

  /**
   * Consructor.
   *
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   Object for file system service.
   * @param \Drupal\File\FileRepositoryInterface $fileRepository
   *   Object for file repository service.
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $streamWrapperManager
   *   Object for stream wrapper manager service.
   * @param \Symfony\Component\Mime\MimeTypeGuesserInterface $guesser
   *   Object for mime type guesser.
   */
  public function __construct(
    FileSystemInterface $fileSystem,
    FileRepositoryInterface $fileRepository,
    StreamWrapperManagerInterface $streamWrapperManager,
    MimeTypeGuesserInterface $guesser
    ) {
    $this->fileSystem = $fileSystem;
    $this->fileRepository = $fileRepository;
    $this->streamWrapperManager = $streamWrapperManager;
    $this->guesser = $guesser;
    // Set default values for variables.
    $this->filterJobId = '';
    $this->capiJobs = [];
    $this->jobsCtt = [];
    $this->capi = new TokenApi();
    $this->statusApi = new StatusUpdateApi();
    $this->requestApi = new RequestApi();
    $this->fileApi = new fileApi();
    $this->jobApi = new JobApi();
    $this->config = new Configuration();
    $this->drupalDateTime = new DrupalDateTime('now', 'UTC');
    $this->translator = $this->setTranslator();
    $this->capiToken = $this->getCapiToken();
    $this->cttToken = $this->getCttToken();
    $this->isAutoAccept = $this->translator->isAutoAccept();
    $this->isAutoImport = $this->isAutoImportSet();
    $this->capiV1Jobs = [];
    if (!empty($this->cttToken)) {
      $this->capiV1Jobs = $this->statusApi->jobsJobIdStatusupdatesGet($this->cttToken);
    }
    $this->capiV2Jobs = $this->statusApi->statusupdatesGet($this->capiToken);
    $this->capiToDeliver = [];
    $this->capiCancelled = [];
    $this->capiToIgnore = [];
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file.system'),
      $container->get('file.repository'),
      $container->get('stream_wrapper_manager'),
      $container->get('file.mime_type.guesser'),
    );
  }

  /**
   * Function to set capi V2 Jobs.
   *
   * @param string $value
   *   Specific job id for porcessing.
   */
  protected function setFilterJobId(string $value) {
    return $this->filterJobId = $value;
  }

  /**
   * Function to set set translator.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   return translator object
   */
  protected function setTranslator() {
    // Get all translators.
    $translators = \Drupal::entityTypeManager()->getStorage('tmgmt_translator')->loadByProperties(['plugin' => 'contentapi']);
    if (!$translators) {
      return [];
    }
    foreach ($translators as $translator) {
      return $translator;
    }

  }

  /**
   * Function to check if auto import set.
   *
   * @return string
   *   capi token
   */
  protected function isAutoImportSet() {
    $config = $this->translator->getSetting('cron-settings');
    return $this->isAutoImport = $config['status'];
  }

  /**
   * Function to get capi token.
   *
   * @return string
   *   capi token
   */
  protected function getCapiToken() {
    $capisettings = $this->translator->getSetting('capi-settings');
    return $this->capi->getToken($capisettings['capi_username_ctt'], $capisettings['capi_password_ctt']);
  }

  /**
   * Function to get ctt token.
   *
   * @return string
   *   ctt token
   */
  protected function getCttToken() {
    $capisettings = $this->translator->getSetting('capi-settings');
    if ($capisettings['capi_username_ctt1'] == '') {
      return '';
    }
    $tokenrequest = new CreateToken(
      [
        'username' => $capisettings['capi_username_ctt1'],
        'password' => $capisettings['capi_password_ctt1']
      ]
    );
    $response = $this->capi->oauth2TokenPost($tokenrequest);
    if (isset($response) != TRUE) {
      \Drupal::logger('TMGMT_CONTENTAPI')->warning("Invalid Content API V1 credentials found while running CRON");
      return '';
    }
    return $response['access_token'];
  }

  /**
   * Function to set scan the updated jobs.
   *
   * @return array
   *   capi jobs of version 1
   */
  protected function scanV1Job() {
    $jobs = [];
    foreach ($this->capiV1Jobs as $value) {
      // Extract each item and put it inside $capiJobs.
      if ($this->filterJobId != '' && $this->filterJobId != $value['job_id']) {
        continue;
      }
      // Create job info array.
      $jobInfo = [];
      $jobInfo['type'] = 1;
      $jobInfo['jobId'] = $value['job_id'];
      $jobInfo['updateId'] = $value['update_id'];
      $jobInfo['requestIds'] = $value['request_ids'];
      $jobInfo['updateTime'] = $value['update_time'];
      $jobInfo['status'] = $value->getStatusCode()->getStatusCode();
      if (!array_key_exists($value['job_id'], $jobsCtt)) {
        $jobsCtt[$value['job_id']] = [];
        $jobsCtt[$value['job_id']][] = $jobInfo;
        continue;
      }
      if ($jobsCtt[$value['job_id']][0]['updateTime'] < $jobInfo['updateTime']) {
        array_unshift($jobsCtt[$value['job_id']], $jobInfo);
      }
      else {
        $jobsCtt[$value['job_id']][] = $jobInfo;
      }
    }
    foreach ($jobsCtt as $value) {
      for ($idx = 0; $idx < count($value); $idx++) {
        if ($idx == 0) {
          $this->capiJobs[$value[$idx]['jobId']] = $value[$idx];
          continue;
        }
        $acknowledge = $this->statusApi->statusupdatesUpdateIdAcknowledgePutCtt($cttToken, $value[$idx]['updateId']);
      }
    }
    return $this->capiJobs;
  }

  /**
   * Function to set scan the updated jobs.
   *
   * @return array
   *   capi jobs of version 2
   */
  protected function scanV2Job() {
    $jobs = [];
    $jobsCtt = [];
    foreach ($this->capiV2Jobs as $value) {
      if ($this->filterJobId != '' && $this->filterJobId != $value['job_id']) {
        continue;
      }
      $jobInfo = [];
      $jobInfo['type'] = 2;
      $jobInfo['jobId'] = $value['job_id'];
      $jobInfo['updateId'] = $value['update_id'];
      $jobInfo['requestIds'] = $value['request_ids'];
      $jobInfo['updateTime'] = $value['update_time'];
      $jobInfo['status'] = $value['status_code'];
      $jobInfo['prevStatus'] = '';
      if (!array_key_exists($value['job_id'], $jobsCtt)) {
        $jobs[$value['job_id']] = [];
        $jobs[$value['job_id']][] = $jobInfo;
        continue;
      }
      if ($jobs[$value['job_id']][0]['updateTime'] < $jobInfo['updateTime']) {
        array_unshift($jobs[$value['job_id']], $jobInfo);
      }
      else {
        $jobs[$value['job_id']][] = $jobInfo;
      }
    }
    foreach ($jobs as $value) {
      for ($idx = 0; $idx < count($value); $idx++) {
        if ($idx == 0) {
          $this->capiJobs[$value[$idx]['jobId']] = $value[$idx];
          continue;
        }
        $acknowledge = $this->statusApi->statusupdatesUpdateIdAcknowledgePut($this->capiToken, $value[$idx]['updateId']);
      }
    }
    return $this->capiJobs;
  }

  /**
   * Load capi jobs and update jobs current status and state on drupal side.
   *
   * @return array
   *   capi jobs with the updated status
   */
  protected function updateJobStatusToLatest() {
    $processedResults = ConentApiHelper::fetchCurrentStoredJobInfo($this->capiJobs);
    foreach ($processedResults as $value) {
      $loadedData = json_decode($value['data'], TRUE);
      if (array_key_exists($loadedData['capiJob']['jobId'], $this->capiJobs) == FALSE) {
        continue;
      }
      $tmpItem = $this->capiJobs[$loadedData['capiJob']['jobId']];
      $tmpItem['providerId'] = $loadedData['capiJob']['providerId'];
      $tmpItem['prevStatus'] = $loadedData['capiJob']['status'];
      // JIRA 1194821: Job status column ambiguity issue.
      $updateTime = $tmpItem['updateTime']->format('Y-m-d H:i:s:u');
      $updateTimeInDB = $loadedData['capiJob']['updateTime'];

      $tmpItem['drupalJob'] = Job::load($value['tjid']);

      if ($tmpItem['prevStatus'] != $tmpItem['status']) {
        if ($tmpItem['prevStatus'] == 'TRANSLATION_APPROVED' && $tmpItem['status'] == 'REVIEW_TRANSLATION') {
          $tmpItem["drupalJob"]->setState(1);
        }
        // Once job is already in REVIEW_TRANSLATION and got some old status skip DB update for the same.
        if ($updateTime > $updateTimeInDB) {
          ConentApiHelper::capiJobSetStoredData(
            $tmpItem['drupalJob'],
            [
              'jobId' => $tmpItem['jobId'],
              'jobStatus' => $tmpItem['status'],
              'providerId' => $tmpItem['providerId'],
              'jobType' => $tmpItem['type'],
              'updateTime' => $updateTime
            ]
          );
        }
      }
      $this->capiJobs[$loadedData['capiJob']['jobId']] = $tmpItem;
    }
    for ($idx = count($this->capiJobs); $idx > 0; $idx--) {
      if (array_key_exists('drupalJob', $this->capiJobs[array_keys($this->capiJobs)[$idx - 1]]) == FALSE) {
        array_splice($this->capiJobs, $idx - 1, 1);
      }
    }
    return $this->capiJobs;
  }

  /**
   * Function to set scan the updated jobs.
   *
   * @param string $jobId
   *   Specific job id which needs to process else it will process all updated jobs.
   */
  protected function scanJob(string $jobId) {
    $this->setFilterJobId($jobId);
    $scan_v1_jobs = [];
    $scan_v2_jobs = [];
    if (!empty($this->capiV1Jobs)) {
      $scan_v1_jobs = $this->scanV1Job();
    }
    if (!empty($this->capiV2Jobs)) {
      $scan_v2_jobs = $this->scanV2Job();
    }
    if (empty($this->capiJobs)) {
      return;
    }
    $this->updateJobStatusToLatest();
  }

  /**
   * Function to filter and aknowledge job status.
   *
   * @param string $jobId
   *   Specific job id which needs to process else it will process all updated jobs.
   *
   * @return array
   *   capi jobs for status updation
   */
  public function filterAknowledgeDownloadJobs(string $jobId) {
    // This functon will filter the job on the basis of current status.
    // Divide jobs into three different array.
    // Deliver: Jobs which are in state of review_translation/trasnlation_approved.
    // Cancelled: Jobs which are in cancelled state and no further processing required.
    // Ignore: Job which are in other than review_translation/translation_approved/cancelled state.

    // Ignore jobs not required to process, acknowledge the job and update job status in DB.

    // Step 1: Scan job for this provider.
    $this->scanJob($jobId);
    // If no job to aknowledge or process then return.
    if (empty($this->capiJobs)) {
      return [];
    }

    // Step 2: Divide job into deliver, cancelled or ignore status.
    $job_to_process = 0;
    foreach ($this->capiJobs as $value) {
      // If the job is not active then ignore it for processing.
      if ($value["drupalJob"]->getState() != 1) {
        $this->capiToIgnore[$value['jobId']] = $value;
        continue;
      }
      switch ($value['status']) {
        case "REVIEW_TRANSLATION":
        case "TRANSLATION_APPROVED":
          $this->capiToDeliver[$value['jobId']] = $value;
          break;

        case "CANCELLED":
          $this->capiCancelled[$value['jobId']] = $value;
          break;

        default:
          $this->capiToIgnore[$value['jobId']] = $value;
      }
    }

    // Step 3: Aknowledge jobs for all statuses.
    // 1: Delivered job.
    // 2. Cancelled job.
    // 0. Ignored job.
    // Aknowledge Jobs which are not ready to deliver.
    $this->aknowledgeJobs($this->capiToIgnore, 0);
    // Aknowledge Jobs which are ready to deliver.
    $this->aknowledgeJobs($this->capiToDeliver, 1);
    // Aknowledge Jobs which have cancelled status.
    $this->aknowledgeJobs($this->capiCancelled, 2);

  }

  /**
   * Function to set akowledge jobs.
   *
   * @param array $capi_jobs_to_aknowledge
   *   Jobs which nees to aknowledge.
   * @param int $job_to_process
   *   Job bifurcation id.
   */
  protected function aknowledgeJobs(array $capi_jobs_to_aknowledge, int $job_to_process) {

    if (empty($capi_jobs_to_aknowledge)) {
      return;
    }
    foreach ($capi_jobs_to_aknowledge as $value) {
      // If update ID is not set then continue with next values.
      if ($value['updateId'] == '') {
        continue;
      }
      // In case of CANCELLED Job.
      if ($job_to_process == 2) {
        // THIS IS TMGMT JOB CANCEL there is no cancel in tmgmt.
        $value['drupalJob']->setState(Job::STATE_ABORTED);
      }

      // In case of job ready for delivery.
      if ($job_to_process == 1) {
        // Step 4: Job which are ready for delivery needs to donwload.
        $this->downloadJob($value);
      }

      // Aknowledge Job.
      try {
        if ($value['type'] == 1) {
          $this->statusApi->statusupdatesUpdateIdAcknowledgePutCtt($this->cttToken, $value['updateId']);
        }
        else {
          $this->statusApi->statusupdatesUpdateIdAcknowledgePut($this->capiToken, $value['updateId']);
        }
      }
      catch (Exception $exception) {
        \Drupal::logger('TMGMT_CONTENTAPI')->error('Could not check delivery list for provider %provider: %message', [
          '%provider' => $translator->id(),
          '%message' => $exception->getMessage(),
        ]);
      }
    }
  }

  /**
   * Function to download Job, auto accept and approve.
   *
   * @param array $job_info
   *   Jobs which nees to download, autoaccept and approve.
   */
  protected function downloadJob(array $job_info) {

    try {
      $job = $job_info['drupalJob'];
      $prId = $job_info['jobId'];
      $job_status = $job_info['status'];
      $request_ids = $job_info['requestIds'];
      $job_from = $job_info['type'];
      $providerId = $job_info['providerId'];
      $dir = $job->getSetting('scheme') . self::PUBLIC_RECEIVED_FILE_PATH;

      // Check if you able to create directory for above mentioned path.
      if (!$this->fileSystem->prepareDirectory($dir, FileSystemInterface::CREATE_DIRECTORY)) {
        \Drupal::messenger()->addMessage(t('Directory could not be created: ') . $dir);
        return;
      }

      $stmrg = $this->streamWrapperManager->getViaUri($dir);
      $tempfolder = $stmrg->realpath();
      $this->config->setTempFolderPath($tempfolder);
      Configuration::setDefaultConfiguration($this->config);
      foreach ($request_ids as $request_id) {
        if ($job_from == 1) {
          $spfilearray = $this->fileApi->jobsJobIdRequestsRequestIdRetrievefileGetWithHttpInfoCtt($this->cttToken, $prId, $request_id);
        }
        else {
          $spfilearray = $this->fileApi->jobsJobIdRequestsRequestIdRetrievefileGetWithHttpInfo($this->capiToken, $prId, $request_id);
        }
        $spfile = $spfilearray[0];
        $fcontent = $spfile->fread($spfile->getSize());
        $fpath = $spfile->getRealPath();
        $spfilearray[0] = NULL;
        $spfile = NULL;
        $this->fileSystem->delete($fpath);
        $filename = str_replace('"', '', explode("=", explode(";", $spfilearray[2]['Content-Disposition'][0])[1])[1]);

        // Step 5: Import the job for review and accept.
        $this->importJob($job, $fcontent, $filename);
      }
      $this->arrayOfRequestId = new ArrayOfRequestIds(['request_ids' => $request_ids]);
      // Verify if all job items are imported.
      // If yes then acknowledge the jobs.
      // Else do now aknowledge Job.

      // If autoaccept set then process the job and send response as TRANSLATION_APPROVED.
      if (!$this->isAutoAccept) {
        return;
      }
      if ($job_from == 1) {
        $returnarray = $this->requestApi->jobsJobIdRequestsApprovePutCtt($this->cttToken, $prId, $this->arrayOfRequestId);
      }
      else {
        $returnarray = $this->requestApi->jobsJobIdRequestsApprovePut($this->capiToken, $prId, $this->arrayOfRequestId);
      }
      // In Last step we have send status "TRANSLATION_APPROVED" to provider.
      // We also need to make sure same status get updated in connector side.
      // JIRA : 1195821 : Job status column ambuguity.
      if (count($returnarray) == 1) {
        $jobstatus = 'TRANSLATION_APPROVED';
        $updateTime = $this->drupalDateTime->format("Y-m-d H:i:s:u");
        $element_array = [
          'jobId' => $prId ,
          'jobStatus' => $jobstatus,
          'providerId' => $providerId,
          'jobType' => $job_from,
          'updateTime' => $updateTime
        ];
        ConentApiHelper::capiJobSetStoredData($job, $element_array);
      }
    }
    catch (Exception $exception) {
      $respbody = $exception->getMessage();
      if ($exception instanceof ApiException) {
        $respbody = $exception->getResponseBody();
      }
      $respbody = 'An error occured while fetching and importing files: ' . $respbody;
      if (strlen($respbody) > 200) {
        $respbody = substr($respbody, 0, 200);
      }
      \Drupal::messenger()->addMessage($respbody, 'error');
    }
  }

  /**
   * Function to import Job.
   *
   * @param \Drupal\tmgmt\Entity\Job $job
   *   Jobs which nees to import.
   * @param string $filedatastring
   *   File content to process.
   * @param string $name
   *   File name to import the job.
   */
  protected function importJob(Job $job, string $filedatastring, string $name) {
    try {
      $message = '';
      $path = $job->getSetting('scheme') . self::PUBLIC_RECEIVED_FILE_PATH . $name;
      $dirname = dirname($path);
      // If not able to create directory return.
      if (!$this->fileSystem->prepareDirectory($dirname, FileSystemInterface::CREATE_DIRECTORY)) {
        return;
      }
      $file = $this->fileRepository->writeData($filedatastring, $path, FileSystemInterface::EXISTS_REPLACE);
      $mimetype = $file->getMimeType();
      $filearray = NULL;
      if ($mimetype == 'application/zip') {
        $ziparchive = new \ZipArchive();
        $openachresult = $ziparchive->open($this->fileSystem->realpath($path));
        if ($openachresult) {
          $pathtoextract = $job->getSetting('scheme') . self::PUBLIC_RECEIVED_FILE_PATH . pathinfo($path)['filename'];
          if ($ziparchive->extractTo($this->fileSystem->realpath($pathtoextract))) {
            $ziparchive->close();
            $file->delete();
            $filearray = $this->cpcreateFileObjectFromStdClass($this->fileSystem->scanDirectory($pathtoextract, '/.*\.xlf$/'));
          }

        }
      }
      else {
        $filearray = [$file];
      }
      foreach ($filearray as $fileitem) {
        $extension = pathinfo($fileitem->getFileUri(), PATHINFO_EXTENSION);
        $plugin = new Xliff();
        if ($plugin) {
          // Validate the file on job.
          $validated_job = $plugin->validateImport($fileitem->getFileUri(), $job);
          if (!$validated_job) {
            \Drupal::logger('TMGMT_CONTENTAPI')->error('Failed to validate file %FILE for job %JOB, import aborted.', [
              '%JOB' => $job->label(),
              '%FILE' => $fileitem->getFileName(),
            ]);
            $job->addMessage('Failed to validate file, import aborted.', [], 'error');
          }
          elseif ($validated_job->id() != $job->id()) {
            \Drupal::logger('TMGMT_CONTENTAPI')->notice('The imported file job id %file_id does not match the job id %job_id.', [
              '%file_id' => $validated_job->id(),
              '%job_id' => $job->id(),
            ]);
            $job->addMessage('The imported file job id @file_id does not match the job id @job_id.', [
              '@file_id' => $validated_job->id(),
              '@job_id' => $job->id(),
            ], 'error');
          }
          else {
            try {
              // Validation successful, start import.
              // TODO: Check if it makes sense to check only the active state here?

              GeneralHelper::resetJobandItemsToActive($job, $fileitem);

              $job->addTranslatedData($plugin->import($fileitem->getFileUri()));
              \Drupal::logger('TMGMT_CONTENTAPI')->notice('Successfully imported file %file_id for job %job_id.', [
                '%file_id' => $fileitem->getFileName(),
                '%job_id' => $job->label(),
              ]);
              $job->addMessage('File @filename imported successfully', ['@filename' => $fileitem->getFileName()]);
            }
            catch (Exception $e) {
              \Drupal::logger('TMGMT_CONTENTAPI')->error('File import for job %JOB failed with the following message: %message', [
                '%JOB' => $job->label(),
                '%message' => $e->getMessage(),
              ]);
              $msg = 'File import failed with the following message: ' . $e->getMessage();
              if (strlen($msg) > 200) {
                $msg = substr($msg, 0, 200);
              }
              $job->addMessage($msg, [], 'error');

            }
          }
        }
      }
    }
    catch (Exception $ep) {
      throw $ep;
    }
  }

  /**
   * Function to cp create file object from stdclass.
   */
  protected function cpcreateFileObjectFromStdClass($filearray) {
        $toreturn = [];
    foreach ($filearray as $fl) {
      $fileMime = $this->guesser->guessMimeType($fl->uri);
      $file = File::create([
      'uid' => \Drupal::currentUser()->id(),
      'filename' => $this->fileSystem->basename($fl->filename),
      'uri' => $fl->uri,
      'filemime' => $fileMime,
      'filesize' => filesize($fl->uri),
      'status' => 1
      ]);
      $file->save();
      array_push($toreturn, $file);
    }
    return $toreturn;
  }

  /**
   * Function to capijob from drupal job.
   *
   * @param \Drupal\tmgmt\Entity\Job $job
   *   Jobs which nees to import.
   */
  public function getCapiJobFromDrupalJob(Job $job) {
    $prId = GeneralHelper::getCpJobIdfromLocJob($job);
    $find_job = $this->jobApi->FindJob($this->capiToken, $prId);
    if ($find_job != 200) {
      $requestsarray = $this->requestApi->jobsJobIdRequestsGetCtt($this->cttToken, $prId);
      $projectInfo = $this->jobApi->jobsJobIdGetCtt($this->cttToken, $prId, "fullWithStats");
      $jobstatus = $projectInfo->getStatusCode()->getStatusCode();
    }
    else {
      $requestsarray = $this->requestApi->jobsJobIdRequestsGet($this->capiToken, $prId);
      $projectInfo = $this->jobApi->jobsJobIdGet($this->capiToken, $prId, "fullWithStats");
      if ($projectInfo->getStatusCode() != 'COMPLETED') {
        $jobstatus = $this->jobApi->jobsJobIdGetStatus($this->capiToken, $prId);
      }
      else {
        $jobstatus = $projectInfo->getStatusCode();
      }
    }
    $requestIds = [];
    foreach ($requestsarray as $request) {
      array_push($requestIds, $request->getRequestId());
    }
    $updateTime = $this->drupalDateTime->format("Y-m-d H:i:s:u");
    $jobInfo['type'] = ($find_job != 200 ? 1 : 2);
    $jobInfo['jobId'] = $projectInfo->getJobId();
    $jobInfo['updateId'] = '';
    $jobInfo['requestIds'] = $requestIds;
    $jobInfo['providerId'] = $projectInfo->getProviderId();
    $jobInfo['status'] = $jobstatus;
    $jobInfo['drupalJob'] = $job;
    $jobInfo['updateTime'] = $updateTime;
    return $jobInfo;
  }

  /**
   * Function to import job via xlf file.
   *
   * @param \Drupal\tmgmt\Entity\Job $job
   *   Jobs which nees to import.
   */
  public function importJobViaFile(Job $job) {
    if ($file = file_save_upload('file', ['file_validate_extensions' => ['xlf']], FALSE, 0)) {
      $extension = pathinfo($file->getFileUri(), PATHINFO_EXTENSION);
      $plugin = new Xliff();
      if ($plugin) {
        // Validate the file on job.
        $validated_job = $plugin->validateImport($file->getFileUri(), $job);
        if (!$validated_job) {
          $job->addMessage('Failed to validate file, import aborted.', [], 'error');
        }
        elseif ($validated_job->id() != $job->id()) {
          $job->addMessage('The imported file job id @file_id does not match the job id @job_id.', [
            '@file_id' => $validated_job->id(),
            '@job_id' => $job->id(),
          ], 'error');
        }
        else {
          try {
            // Validation successful, start import.
            // Check if it makes sense to check only the active state here?
            GeneralHelper::resetJobandItemsToActive($job, $file);
            $job->addTranslatedData($plugin->import($file->getFileUri()));
            $job->addMessage('Successfully imported file.');
            ConentApiHelper::checkJobFinishAndApproveRemote($job);
          }
          catch (Exception $e) {

            $msg = 'File import failed with the following message: ' . $e->getMessage();
            if (strlen($msg) > 200) {
              $msg = substr($msg, 0, 200);
            }
            $job->addMessage($msg, [], 'error');
          }
        }
        tmgmt_write_request_messages($job);
      }
    }

  }

}
